
/******************************************************************************* 
 * File:   RX_HT6P_XC8.C
 * Autor : Cludio Lrios
 * Data  : 21/07/2018
 * Receptor de controle remoto rf usando integrado decodificado HT6P20B 
 * Pic 12F675 ou 12f629
 * Receptor monocanal operando em modo pulso (aciona sada enquanto controle for
 * acionado pelo usurio)
 * Use compilador Mplab X IDE v3.05
 * 
 * Gravar controles remotos novos:
 * Aperte boto sw_learn at ligar o led learn e solte o boto.
 * Aperte um dos botes do controle remoto at o led_learn apagar.
 * Ao gravar dever acionar a sada.
 * Ateno: Grave cada boto do controle que deseja usar. Botes no gravados
 * no funcionaro!
 * 
 * Apagar todos controles gravados:
 * Aperte o boto sw_learn por mais de 5 segundos.
 * Aps este tempo, o led_learn piscar por 10 vezes para indicar sucesso no 
 * apagamento  dos controles remotos na eeprom. 
 * 
 * Indicao de sinal HT6P20B presente:
 * O led learn ir piscar de forma fraca e rpida para indicar que algum 
 * controle remoto foi acionado nas prximidades do receptor, mesmo para tx
 * no gravados ou desconhecidos.
 * Grava at 34 botes de controles remotos diferentes
 * 
 * Blog Picsource
******************************************************************************/
#include <xc.h>
#define _XTAL_FREQ 4000000//frequncia usada (osc.interno))

__CONFIG(PWRTEN & WDTDIS & INTIO & CP_OFF & MCLRDIS);    //fuses

//DEFINES PARA AS PINAGENS DO PIC
#define led_learn   GPIO5 //gp5 - pino 2 = led 'learn' ao gnd.
#define rf          GPIO4 //gp4 - pino 3 = in_rf (entrada do receptor de rf 433mhz)
#define sw_learn    GPIO3 //gp3 - pino 4 = interruptor 'learn' ao gnd
#define out         GPIO2 //gp2 - pino 5 = sada controlada remotamente 

//DEFINES DE CONSTANTES
#define NUM_MAX_CONTROLES 34 //quantidade mxima de controles remotos com 3 
                             //botes que podem ser gravados na eeprom do pic.
#define TEMPO_APAGAMENTO 5  //tempo (seg) que sw_learn precisa ficar apertado p/ 
                            //apagar todos os controles da memria.

#define TEMPO_OUT_DESL  15  //fator - tempo de desligamento da sada (pulso).
#define TEMPO_LED_APRENDER  250//fator - tempo que o led_learn fica aceso aps 
                            //apertar interruptor sw_learn.
#define  TIME_8MS     -8000 //tempo de 8000us usado na deteo de pausa 
                            //na transmisso.
#define  TIME_PULSO_B -700  //largura mxima do pulso bsico (em us).
#define VALOR_MINIMO_PB  200//largura mnima do pulso bsico (em us).
#define N_CONTROL (NUM_MAX_CONTROLES * 3+1)//define auxiliar
#define T_APAG  TEMPO_APAGAMENTO * 1000//define auxiliar

// VARIAVIS GLOBAIS
unsigned long  dado=0;
unsigned char timer5 = TEMPO_OUT_DESL;
unsigned char timer4 = TEMPO_LED_APRENDER;
unsigned char timer6 =0;
unsigned char pilha=0;
unsigned char a;
unsigned short timer_save=0;
unsigned short meio_periodo=0;
bit salva_led;
//PR-GRAVAO DA EEPROM (NA GRAVAO DO PIC)
__EEPROM_DATA(0,0xff,0xff,0xff,0xff,0xff,0xff,0xff);

/******************************************************************************
 SUB-ROTINAS 
 ******************************************************************************/ 


/******************************************************************************
 ROTINA PREPARATRIA DO TEMPO A SER TEMPORIZADO NO TMR1
*******************************************************************************/
void temporizar(unsigned short tempo){
    TMR1ON=0;//desliga tmr1
    TMR1L=tempo;//carga de tempo low
    TMR1H=tempo>>8;//carga de tempo high
    TMR1IF=0;// reseta flag de overflow
    TMR1ON=1; //liga tmr1   
}
/******************************************************************************
    ROTINA DE SUBTRAO DO ENDEREO DA EEPROM
 ******************************************************************************/
void subt3(){    
  if(pilha>=3)pilha-=3;else pilha=0;//subtrai 3   
}
/******************************************************************************
    ROTINA DE LEITURA DA EEPROM
 ******************************************************************************/
unsigned char le_eeprom (unsigned char end){
    return eeprom_read(end);    
}
/******************************************************************************
    ROTINA DE ESCRITA DA EEPROM
 ******************************************************************************/
void escreve_eeprom(unsigned char ender,unsigned char valor){    
    eeprom_write(ender,valor);    
}
/******************************************************************************
    ROTINA DE PROCURA DE CONTROLE REMOTO GRAVADO NA EEPROM
 ******************************************************************************/
unsigned char procurar_controle(){
     pilha=le_eeprom(0);//le posio que guarda o nmero de controles gravados      
     while(pilha>0){ //procura a partir do topo da pilha para baixo      
      if(le_eeprom(pilha)!= (dado& 0xff)){subt3();continue;}      
      if(le_eeprom(pilha-1)!= (dado>>8 & 0xff)){subt3();continue;}     
      if(le_eeprom(pilha-2)!= (dado>>16 & 0xff)){subt3();continue;}      
      return 0xff;  //controle encontrado na eeprom        
    }    
    return 0; //controle no encontrado na eeprom
}
/******************************************************************************
    ROTINA DE RECEPO DO SINAL HT6P20B
 ******************************************************************************/
void recebe_ht6p(){    
    
    //sincronizando recepo (pausa de 23 ciclos de clock c/ a entrada em '0')
    
    
    temporizar(TIME_8MS);//carrega timer1 para tempo de 8ms
    while(!TMR1IF){//vigia flag de overflow
        if(rf) return;//intervalo de pausa curto demais
    } 
        
    temporizar(TIME_8MS);//carrega timer1 para tempo de 8ms
     while(!rf){//vigia entrada rf
        if(TMR1IF) return;//intervalo de pausa longo demais
    } 
        
    //recebendo pulso bsico ('1')
   
    temporizar(TIME_PULSO_B);//carrega timer1 com tempo p/ 'pulso bsico'(700us)
    while(rf){//vigia entrada rf
      if(TMR1IF)return;  //erro! Pulso base longo demais
    }
    
    //recebeu pulso bsico passando para o primeiro bit composto 0|x|1
    //salva antes o valor do timer 1 com valor do pulso bsico  
    
    TMR1ON=0;  //desliga tmr1 
    timer_save=TMR1L;//salva valor low
    timer_save|=(short)TMR1H<<8;//salva valor high       
    timer_save-= (TIME_PULSO_B);//obtem a contagem real do pulso     
    if(timer_save<VALOR_MINIMO_PB)return; //descarta pulsos de interferencias   
    meio_periodo=timer_save;//copia valor do pulso bsico real
    meio_periodo= meio_periodo>>1;//divide por 2     
    meio_periodo=~meio_periodo;//inverte
    timer_save=~timer_save;//inverte
  
    
    //rotina para receber 24 bits do tx com ci HT6P20b
    dado=0;
    for(a=0;a<24;a++){
        temporizar(meio_periodo);//carrega metade do valor do pulso bsico
        while(!TMR1IF);//aguarda completar o tempo
        if(rf)return;//erro
    
        temporizar(timer_save);//carrega valor do pulso bsico
        while(!TMR1IF);//aguarda completar o tempo
        dado=dado>>1;//desloca para direita um bit
        if(!rf)dado=dado|0x00800000;//salva valor do bit recebido em 'dado'
     
        temporizar(timer_save);//carrega valor do pulso bsico
        while(!TMR1IF);//aguarda completar o tempo
        if(!rf)return;//erro
        if(led_learn)salva_led=1;else salva_led=0;//recurso para pisca led ao
        if(salva_led)led_learn=0; else led_learn=1;//receber sinal no pino rf
        while(rf);//aguarda rf ficar '0'
        if(salva_led)led_learn=1; else led_learn=0;//retorna estado do led_learn
    }
        
    // procurar controle na eeprom interna do pic, se foi gravado
    if(procurar_controle()){        
       out=1; //sada  acionada pois foi encontrado um boto gravado
       timer5 = TEMPO_OUT_DESL;//nova carga de tempo para impedir desligamento
       led_learn=0;//apaga led de programao tambm
    }
    else{
        if(led_learn){//boto 'programar' foi acionado (ir aprender controle)
            pilha=le_eeprom(0);//busca posio que dever ser gravado na eeprom
            
            if(pilha+4<=N_CONTROL){//grava controle novo na eeprom
                escreve_eeprom(pilha+1,dado>>16);
                escreve_eeprom(pilha+2,dado>>8);
                escreve_eeprom(pilha+3,dado);
                pilha+=3;//altera valor da posio livre 
                escreve_eeprom(0,pilha);//salva no stack
            } 
            led_learn=0;//apaga led learn
        }        
    }    
}
/******************************************************************************
 ROTINA DE TEMPORIZAO USANDO TIMER 0 
 * DESLIGAMENTO DA SADA
 * DESLIGAMENTO DO MODO DE PROGRAMAO
 ******************************************************************************/ 
void time_out(){
    if(!T0IF)return;
    T0IF = 0;
    if(timer4)timer4--;//
    if(timer5)timer5--;
    
    if((!timer5)&&(out)){
        out=0;//desliga sada por tempo
        timer5=TEMPO_OUT_DESL;//nova carga de tempo
    }  
    if(!led_learn)return; //se no est em modo programao sai da rotina   
    if(!timer4){//se ='0'
       led_learn=0;//apaga led 'learn' por tempo
       timer4=TEMPO_LED_APRENDER; //nova carga de tempo       
    }
}
/******************************************************************************
 ROTINA DE TESTE DO INTERRUPTOR 'SW' (MODO PROGRAMAO) 
 ******************************************************************************/ 
void sw_on(){
    unsigned short timer7=T_APAG;
    unsigned char a;
    if(!sw_learn){
        led_learn=1;
        while(!sw_learn){
            __delay_ms(1);
            timer7--;
            if(!timer7){
                for(a=1;a<128;a++){
                   escreve_eeprom(a,0xff) ;
                }
                escreve_eeprom(0,0);
                for(a=0;a<10;a++){//pisca led learn por 10 vezes
                led_learn=1;
                __delay_ms(200);
                led_learn=0;
                __delay_ms(200);
                }
                while(!sw_learn);//aguarda soltar boto
            }
        }
    }   
}

/******************************************************************************
                         PROGRAMA MAIN
 ******************************************************************************/ 
    main(void){
	
    CMCON=7; //Desliga comparador interno
	// configure IO pins
	TRISIO = 0b011011;// Sentido dos pinos
    GPIO5 = 0;	// desliga led
    GPIO2 = 0;	// desliga sada comandada	

#ifdef _12F675
	ANSEL = 0;
#endif
	// timer 0 (PWM period generator)
	OPTION_REG = 0xD7;		// 0b 1101 0111  T0CS=0, PSA=0, 1:256   
	T0IF = 0;
    T1CON=0b00000001;//liga timer 1  presc=/1
    timer5=TEMPO_OUT_DESL;
    timer4=TEMPO_LED_APRENDER;
    
        
/******************************************************************************
	LAO PRINCIPAL DE REPETIO
 ******************************************************************************/   
	for(;;){
      recebe_ht6p();//rotina de recepo  
      time_out();//rotina de temporizao da sada e do modo de programao 
      sw_on();//rotina de teste do interruptor 'learn'		
	}
}
    
/*******************************************************************************
 *                 FIM DA ROTINA DE RECEPO
 *                  BLOG PICSOURCE
 ******************************************************************************/